#ifndef PSimHit_H
#define PSimHit_H

#include "LocalPoint.h"
#include "LocalVector.h"
// ROOT addition
#include "TObject.h"

class DetUnit;

/** Persistent-capable SimHit.
 *  Suitable for tracking detectors.
 */

// ROOT change
class PSimHit : public TObject {
public:

  PSimHit() : theDetUnitId(-1) {}

  PSimHit( const Local3DPoint& entry, const Local3DPoint& exit, 
	   float pabs, float tof, float eloss, int particleType,
	   int detId, unsigned int trackId) :
  theEntryPoint( entry), theExitPoint(exit),
  thePabs(pabs), theTof(tof), theEnergyLoss(eloss),
  theParticleType(particleType),
  theDetUnitId( detId), theTrackId( trackId) 
{}

  Local3DPoint entryPoint()      const {return theEntryPoint;}
  Local3DPoint exitPoint()       const {return theExitPoint;}
  Local3DPoint localPosition()   const {
    return entryPoint() + 0.5* (exitPoint() - entryPoint());
  }
  LocalVector  localDirection()  const {
    return (exitPoint() - entryPoint()).unit();
  }

  float        pabs()            const {return thePabs;}
  float        tof()             const {return theTof;}
  float        energyLoss()      const {return theEnergyLoss;}
  int          particleType()    const {return theParticleType;}

  int          detUnitId()       const {return theDetUnitId;}
  unsigned int trackId()         const {return theTrackId;}

private:
  
  // properties
  Local3DPoint    theEntryPoint;    // position
  Local3DPoint    theExitPoint;
  float           thePabs;          // momentum
  float           theTof;           // Time Of Flight 
  float           theEnergyLoss;    // Energy loss
  int             theParticleType;

  // association
  int             theDetUnitId;
  unsigned int    theTrackId;
  
// ROOT addition
  ClassDef(PSimHit,1)

};

   
#include<iostream>
inline std::ostream & operator<<(std::ostream & o, const PSimHit & hit) {
  return o << hit.detUnitId() 
	   << " " << hit.entryPoint() 
	   << " " << hit.tof();
}

#endif // PSimHit_H


