#ifndef Point3DBase_H
#define Point3DBase_H


#include "PointTag.h"
#include "PV3DBase.h"
#include "Point2DBase.h"
#include "Vector3DBase.h"

template <class T, class FrameTag>
class Point3DBase : public PV3DBase< T, PointTag, FrameTag> {
public:

  typedef PV3DBase< T, PointTag, FrameTag>    BaseClass;
  typedef Vector3DBase< T, FrameTag>          VectorType;
  typedef Basic3DVector<T>                    BasicVectorType;

  Point3DBase() {}

  Point3DBase(const T& x, const T& y, const T& z) : BaseClass(x, y, z) {}

  // mimick 2D point
  Point3DBase( const T& x, const T& y) : BaseClass( x, y, 0) {}
  explicit Point3DBase( Point2DBase<T,FrameTag> p) : 
    BaseClass( p.x(), p.y(), 0) {}

  explicit Point3DBase( const BasicVectorType& v) : BaseClass(v) {}

  Point3DBase& operator+=( const VectorType& v) {
    basicVector() += v.basicVector();
    return *this;
  } 

  Point3DBase& operator-=( const VectorType& v) {
    basicVector() -= v.basicVector();
    return *this;
  } 
  
};

template<class T, class F>
Point3DBase<T,F> operator+( const Point3DBase<T,F>& p,
			    const Point3DBase<T,F>::VectorType& v) {
  return Point3DBase<T,F>(p) += v;
}

template<class T, class F>
Point3DBase<T,F> operator+( const Point3DBase<T,F>::VectorType& v,
			    const Point3DBase<T,F>& p) {
  return p + v;
}



template<class T, class F>
Point3DBase<T,F>::VectorType operator-( const Point3DBase<T,F>& p1,
					const Point3DBase<T,F>& p2) {
  return Point3DBase<T,F>::VectorType( p1.basicVector() - p2.basicVector());
}

#endif // Point3DBase_H
